﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Drawing;
using System.Text;
using System.Text.RegularExpressions;
using System.Net;
using System.IO;
using System.Diagnostics;
using System.Reflection;
using System.CodeDom.Compiler;
using Microsoft.JScript;
using LiveViewer;
using LiveViewer.Tool;
using LiveViewer.HtmlParser;

namespace Plugin_BBChat {
    public class Plugin_BBChat : ISitePlugin {
        #region ■定義

        private class FormFlashBBChat : FormFlash {
            public Timer  Timer   = new Timer(); //メッセージ定期チェック用
            public string Message = null;        //メッセージ(前回との比較用)

            public FormFlashBBChat(Performer pef)
                : base(pef) {
                FormClosed  += new FormClosedEventHandler(FormFlash_FormClosed);
                FlashLoaded += new FormFlash.FormFlashEventHandler(FormFlash_FlashLoaded);
                Timer.Tick  += new EventHandler(FormFlash_Timer_Tick);
            }

            private void FormFlash_FormClosed(object sender, FormClosedEventArgs e) {
                Timer.Dispose();
            }

            private void FormFlash_FlashLoaded(FormFlash ff) {
                Timer.Interval = 1000;
                Timer.Enabled  = true;
            }

            private void FormFlash_Timer_Tick(object sender, EventArgs e) {
                //メッセージ取得
                try {
                    string sMes = FlashGetVariable("_root.msg_board_txt.text");
                    if (sMes != null && sMes != "" && sMes != Message) {
                        Message = sMes;
                        Log.AddMessage(Performer, sMes); //メッセージログ表示
                    }
                } catch {
                }
            }
        }

        #endregion


        #region ■オブジェクト

        private Regex     RegexGetSwf    = new Regex("<param name=\"movie\" value=\"([^\"]*)\"", RegexOptions.Compiled);
        private WebClient Client         = new WebClient(); //HTML取得用

        private Type   JsExecuterType   = null;
        private object JsExecuterObject = null;
        private string JsSource         = @"
            package JSExecuter
            {
                public class JSExecuter {
                    public function Eval(sJsCode : String) : Object { 
                        return eval(sJsCode);
                    }
                }
            }
        ";

        #endregion


        #region ■ISitePluginインターフェースの実装

        public string Site       { get { return "bb-chat"; } }

        public string Caption    { get { return "BBChat用のプラグイン(2011/06/21版)"; } }

        public string TopPageUrl { get { return "http://bb-chat.tv/"; } }

        public void Begin() {
            //プラグイン開始時処理

            //コンパイルするための準備
            JScriptCodeProvider jcp = new JScriptCodeProvider();

            //コンパイルパラメータ（メモリ内で生成）
            string[] assemblys = new string[] { Assembly.GetAssembly(this.GetType()).Location};
            CompilerParameters cp = new CompilerParameters(assemblys);
            cp.GenerateInMemory = true;

            //コンパイル
            CompilerResults cres = jcp.CompileAssemblyFromSource(cp, JsSource);

            //コンパイルしたアセンブリを取得
            Assembly asm = cres.CompiledAssembly;

            //クラスのTypeを取得
            JsExecuterType = asm.GetType("JSExecuter.JSExecuter");

            //インスタンスの作成
            JsExecuterObject = Activator.CreateInstance(JsExecuterType);

            //User-Agentを設定
            Client.Headers.Add(HttpRequestHeader.UserAgent, Pub.UserAgent + "_" + Site);
        }

        public void End() {
            //プラグイン終了時処理
        }

        public List<Performer> Update() {
            List<Performer> pefs = new List<Performer>();

            try {
                //WebからJsファイルを読み取る
                StringBuilder sb = new StringBuilder();
                using (Stream s = Client.OpenRead(TopPageUrl + "ajax/get_online_girls_list.php")) {
                    using (StreamReader sr = new StreamReader(s, Encoding.GetEncoding("Shift-JIS"))) {
                        sb.AppendLine(sr.ReadToEnd());
                    }
                }
                string sJson = sb.ToString().Split('\n')[0] + ";";

                //Jsファイルの内容を実行する
                ArrayObject obj = JsExecuterType.InvokeMember("Eval", BindingFlags.InvokeMethod, null, JsExecuterObject, new object[] { sJson }) as ArrayObject;
                for (int i = 0; i < (int)obj.length; i++) {
                    JSObject  jso = obj[i] as JSObject;
                    string    sID = jso.GetField("hash", BindingFlags.Default).GetValue(null) as string;
                    Performer p = new Performer(this, sID);
                    p.Name             = HttpUtilityEx.HtmlDecode((string)jso.GetField("nick_name", BindingFlags.Default).GetValue(null));
                    p.ImageUrl         = jso.GetField("img", BindingFlags.Default).GetValue(null) as string;
                    p.ImageUpdateCheck = false;

                    string sStatus = (string)jso.GetField("chat_stat", BindingFlags.Default).GetValue(null);
                    switch (sStatus) {
                        case "オンライン": p.Dona = false; p.TwoShot = true; break;
                        case "チャット中": p.Dona = true;  p.TwoShot = true; break;
                        default: Log.Add(Site + "-ERROR", "不明な状態画像1:" + sStatus, LogColor.Error); break;
                    }

                    string sImg = (string)jso.GetField("chatimg", BindingFlags.Default).GetValue(null);
                    switch (sImg) {
                        case "/images/online_icon/top-busy.gif":
                        case "/images/online_icon/top-ok.gif":         break;
                        case "/images/online_icon/top-busy-today.gif":
                        case "/images/online_icon/top-ok-today.gif":   p.Debut = true; break;
                        case "/images/online_icon/top-busy-new.gif":
                        case "/images/online_icon/top-busy-new2.gif":
                        case "/images/online_icon/top-busy-new3.gif":
                        case "/images/online_icon/top-ok-new.gif":
                        case "/images/online_icon/top-ok-new2.gif":
                        case "/images/online_icon/top-ok-new3.gif":    p.NewFace = true; break;
                        default: Log.Add(Site + "-ERROR", "不明な状態画像2:" + sStatus + ":" + sImg, LogColor.Error); break;
                    }

                    pefs.Add(p);
                }
                return pefs;
            } catch (Exception ex) {
                Log.Add(Site + "-Update失敗", ex.ToString(), LogColor.Error);
                return null;
            }
        }

        public FormFlash OpenFlash(Performer performer) {
            //フラッシュ窓を返す
            return new FormFlashBBChat(performer);
        }
        
        public string GetFlashUrl(Performer performer) {
            //FlashのURLを返す・・待機画像ページのHTMLから取得する
            string sFlash = null;
            try {
                using (WebClient wc = new WebClient()) {
                    wc.Headers.Add(HttpRequestHeader.UserAgent, Pub.UserAgent + "_" + Site);
                    string sHtml = wc.DownloadString(TopPageUrl + "online.php?id=" + performer.ID);
                    sFlash = TopPageUrl + RegexGetSwf.Match(sHtml).Groups[1].Value;
                    Pub.WebRequestCount++; //GUIの読込回数を増やす
                }
            } catch (Exception ex) {
                Log.Add(Site + "-GetFlashUrl失敗", ex.ToString(), LogColor.Error);
            }
            return sFlash;
        }

        public Clipping GetFlashClipping(Performer performer) {
            //Flashの切り抜き方法を返す
            return null;
        }

        public string GetProfileUrl(Performer performer) {
            //プロフィールURLを返す
            //プロフィールだけの画面：return TopPageUrl + "girl-syosai.php?girl=" + performer.ID;
            return TopPageUrl + "online.php?s=1&id=" + performer.ID;
        }

        #endregion
    }
}
