﻿#define SHOW_MOBILE //★この行を削除すればマシェリモバイルのパフォは表示されません

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Drawing;
using System.Text;
using System.Text.RegularExpressions;
using System.Net;
using System.Diagnostics;
using LiveViewer;
using LiveViewer.Tool;
using LiveViewer.HtmlParser;

namespace Plugin_Macherie {
    public class Plugin_MacherieX : ISitePlugin {
        #region ■定義

        private class FormFlashMacherie : FormFlash {
            public Timer  Timer   = new Timer(); //メッセージ定期チェック用
            public string Message = null;        //メッセージ(前回との比較用)
            public FormFlashMacherie(Performer pef)
                : base(pef) {
                FormClosed              += new FormClosedEventHandler(FormFlash_FormClosed);
                FlashLoaded             += new FormFlash.FormFlashEventHandler(FormFlash_FlashLoaded);
                Timer.Tick              += new EventHandler(FormFlash_Timer_Tick);
            }

            private void FormFlash_FormClosed(object sender, FormClosedEventArgs e) {
                Timer.Dispose();
            }

            private void FormFlash_FlashLoaded(FormFlash ff) {
                Timer.Interval = 1000;
                Timer.Enabled  = true;
            }

            private void FormFlash_Timer_Tick(object sender, EventArgs e) {
                //メッセージ取得
                try {
                    string sMes = FlashGetVariable("main_mc.girl_msg1.text");
                    if (sMes != null && sMes != "" && sMes != Message) {
                        Message = sMes;
                        Log.AddMessage(Performer, sMes); //メッセージログ表示
                    }
                } catch {
                }
            }
        }

        #endregion


        #region ■オブジェクト

        private Parser    Parser   = new Parser();    //HTML解析用パーサ

        //HTML解析用の正規表現
        private Regex RegexGetPef = new Regex(" gB", RegexOptions.Compiled);
        private Regex RegexGetSwf = new Regex("<param name=\"movie\" value=\"/([^\"]*)\"", RegexOptions.Compiled);

        private bool  UseEmergencyUrl = false; //非常用URLを使うかどうか

        #endregion


        #region ■ISitePluginインターフェースの実装

        public string Site       { get { return "macherie"; } }

        public string Caption    { get { return "マシェリ用のプラグイン(2011/06/21版)"; } }

        public string TopPageUrl { get { return UseEmergencyUrl ? "http://sr.macherie.tv/" : "http://macherie.tv/"; } }

        public void Begin() {
            //プラグイン開始時処理

            //User-Agentを設定
            Parser.UserAgent = Pub.UserAgent + "_" + Site;
        }

        public void End() {
            //プラグイン終了時処理
        }

        public List<Performer> Update() {
            //オンラインパフォーマーリスト生成
            List<Performer> pefs = new List<Performer>();

            try {
                //トップページの読み込みと木構造への変換
                Parser.LoadHtml(new Uri(TopPageUrl));
                Parser.ParseTree();
                Pub.WebRequestCount++;
            } catch (WebException ex){
                //読み込み失敗(Web関連)
                UseEmergencyUrl = !UseEmergencyUrl;
                Log.Add(Site + "-Update失敗(Web)", "次回の読み込み先を" + (UseEmergencyUrl ? "非常用" : "通常") + "URLに変更します。" + ex.ToString(), LogColor.Error);
                return null;
            } catch (Exception ex) {
                //読み込み失敗(その他)
                Log.Add(Site + "-Update失敗", ex.ToString(), LogColor.Error);
                return null;
            }

            //パフォ情報のタグを取得
            List<HtmlItem> tagTops = Parser.Find("ul", "class", RegexGetPef);
            Parser.Clear();

            //各パフォの情報を順に処理
            foreach (HtmlItem tagTop in tagTops) {
                string sOut;
                if (tagTop.Parent.TryGetAttributeValue("id", out sOut)) {
                    if (sOut == "newFace") {
                        continue; //新人紹介なので読み飛ばす
                    }
                }

                //IDと名前の取得
                HtmlItem tagName = tagTop.Find("li", "class", "name", false)[0];
                string   sID     = tagName.Items[0].GetAttributeValue("href").Split('\'')[1];

                //オブジェクト作成
                Performer p = new Performer(this, sID);

                //名前の取得
                string sName = tagName.Items[0].Items[0].Text;
                p.Name = HttpUtilityEx.HtmlDecode(sName);
                if (p.Name == null) {
                    //エラー対策
                    if (tagName.Items[0].Items[0].Type == HtmlItemType.Tag) {
                        p.Name = tagName.Items[0].Items[0].ToString();
                    } else {
                        p.Name = "不明";
                    }
                }

                //状態の取得
                string sTmp = tagTop.GetAttributeValue("class");
                switch (sTmp) {
                    case "on gB"  : p.Dona = false; p.TwoShot = true;  p.RoomName = "2ｼｮｯﾄ";  break;
                    case "ch gB"  : p.Dona = true;  p.TwoShot = true;  p.RoomName = "2ｼｮｯﾄ";  break;
                    case "pon gB" : p.Dona = false; p.TwoShot = false; p.RoomName = "ﾊﾟｰﾃｨｰ"; break;
                    case "pc gB"  : p.Dona = true;  p.TwoShot = false; p.RoomName = "ﾊﾟｰﾃｨｰ"; break;
#if SHOW_MOBILE
                    case "mb gB"  : p.Dona = true;  p.TwoShot = true;  p.RoomName = "ﾓﾊﾞｲﾙ";  p.Mobile = true; break;
#else
                    case "mb gB"  : continue; //モバイルなので登録しない
#endif
                    case "cm gB"  : continue; //CM(?)なので登録しない
                    case "off gB" : continue; //オフラインなので登録しない
                    default       : Log.Add(Site + "-不明な状態:", sTmp, LogColor.Error); break;
                }

                //部屋の取得
                string sRoom = null;
                tagTop.Parent.Parent.TryGetAttributeValue("id", out sRoom);
                if (sRoom == "roomArea") {
                    p.RoomName = "ｲﾍﾞﾝﾄ";
                }

                //マイクの取得
                if(tagName.Find("img", "alt", "voice", true).Count > 0){
                    p.Mic = true;
                }

                //新人の取得
                HtmlItem tagOpt = tagTop.Find("li", "class", "tag", false)[0];
                bool bFineDebut = tagOpt.Find("img", "alt", "高画質新人", true).Count > 0;
                if (tagOpt.Find("img", "alt", "new", true).Count > 0) {
                    p.NewFace = true;
                } else if (tagOpt.Find("img", "alt", "debut", true).Count > 0 || bFineDebut) {
                    p.Debut = true;
                }

                //高画質マークの取得
                bool bFine = tagOpt.Find("img", "alt", "高画質", true).Count > 0;
                if (bFine || bFineDebut) {
                    p.OtherInfo += "高画質 ";
                }

                //チェックマークの取得
                if (tagOpt.Find("img", "alt", "チェックマーク", true).Count > 0) {
                    p.OtherInfo += "CHECK ";
                }

                //画像URLの取得
                string sImgUrl = tagTop.Find("li", "class", "pic", false)[0].GetAttributeValue("style").Split('(', ')')[1];
                if (sImgUrl[0] == 'h') {
                    p.ImageUrl = sImgUrl;
                } else {
                    p.ImageUrl = TopPageUrl + sImgUrl;
                }
                p.ImageUpdateCheck = false;

                pefs.Add(p);
            }

            return pefs;
        }

        public FormFlash OpenFlash(Performer performer) {
            //フラッシュ窓を返す
            if (performer.RoomName == "ﾓﾊﾞｲﾙ") {
                MessageBox.Show("モバイルなので待機映像はありません");
                return null;
            } else {
                return new FormFlashMacherie(performer);
            }
        }

        public string GetFlashUrl(Performer performer) {
            //FlashのURLを返す・・待機画像ページのHTMLから取得する
            string sFlash = null;
            try {
                using (WebClient wc = new WebClient()) {
                    wc.Headers.Add(HttpRequestHeader.UserAgent, Pub.UserAgent + "_" + Site);
                    string sHtml = wc.DownloadString(TopPageUrl + "chat/shicho.php?id=" + performer.ID);
                    sFlash = TopPageUrl + RegexGetSwf.Match(sHtml).Groups[1].Value;
                    Pub.WebRequestCount++; //GUIの読込回数を増やす
                }
            } catch (WebException ex){
                //読み込み失敗(Web関連)
                UseEmergencyUrl = !UseEmergencyUrl;
                Log.Add(Site + "-GetFlashUrl失敗(Web)", "次回の読み込み先を" + (UseEmergencyUrl ? "非常用" : "通常") + "URLに変更します。" + ex.ToString(), LogColor.Error);
            } catch (Exception ex) {
                //読み込み失敗(その他)
                Log.Add(Site + "-GetFlashUrl失敗", ex.ToString(), LogColor.Error);
            }
            return sFlash;
        }

        public Clipping GetFlashClipping(Performer performer) {
            //Flashの切り抜き方法を返す
            Clipping c = new Clipping();
            c.OriginalSize.Width  = 792;  //フラッシュ全体の幅
            c.OriginalSize.Height = 506;  //フラッシュ全体の高さ
            c.ClippingRect.X      = 12;   //切り抜く領域の左上座標(X)
            c.ClippingRect.Y      = 49;   //切り抜く領域の左上座標(Y)
            c.ClippingRect.Width  = 596;  //切り抜く領域の幅
            c.ClippingRect.Height = 447;  //切り抜く領域の高さ
            c.Fixed               = true; //Flashが固定サイズ
            return c;
        }

        public string GetProfileUrl(Performer performer) {
            //プロフィールURLを返す
            //プロフィールだけの画面：return TopPageUrl + "profile/profile.php?sid=" + performer.ID;
            return TopPageUrl + "chat/shicho.php?id=" + performer.ID;
        }

        #endregion
    }
}
